<?php

namespace App\Http\Controllers\Admin;

use App\Message;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

class AdminMessageController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    public function index()
    {
        $messages = Message::orderBy('is_read', 'asc')->orderBy('created_at', 'desc')->paginate(15);

        return view('admin.messages', compact('messages'))->with(['controller' => $this]);
    }

    public function store(Request $request)
    {

        $this->validate($request, [
            'email' => 'email|max:100',
            'name' => 'max:255|string',
            'phone' => 'numeric|max:100000000000000',
            'message' => 'required|string|max:10000',
        ]);

        $message = new Message();
        $message->email = $request->email;
        $message->name = $request->name;
        $message->phone = $request->phone;
        $message->content = $request->message;
        if ($message->save()) {
            return back()->with('success', 'تم إضافة إرسالة رسالتك بنجاح..');
        }
    }

    public function delete(Request $request)
    {
        if (!$request->id) {
            return 'معرف الرسالة غير صالح';
        }
        $item = Message::find($request->id);
        if ($item != null) {
            if ($item->delete()) {
                return "تم عملية الحذف بنجاح";
            } else {
                return "حدث خطأ غير متوقع أثناء الحذف .. حاول في وقت آخر !";
            }
        } else {
            return " تحاول حذف عنصر غير موجود ..  !";
        }
    }

    public function read(Request $request)
    {
        if (!$request->id) {
            return 'معرف الرسالة غير صالح';
        }
        $item = Message::find($request->id);
        if ($item != null) {
            $item->is_read = 1;
            if ($item->save()) {
                return "تم تعيين الرسالة كمقروءة";
            } else {
                return "حدث خطأ غير متوقع أثناء قراءة الرسالة .. حاول في وقت آخر !";
            }
        } else {
            return " تحاول قراءة عنصر غير موجود ..  !";
        }
    }
}
