<?php

namespace App\Http\Controllers\Admin;

use App\Subscription;
use App\SubscriptionPlan;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;

class AdminSubscriptionController extends Controller
{
    public function index()
    {
        $items = Subscription::orderBy('created_at', 'desc')->paginate(15);
        $title = "إشتراكات المغاسل الفعالة ";
        return view('subscriptions', compact('items', 'title'))->with(['controller' => $this]);
    }

    public function expired()
    {
        $timestamp = time();
        $items = Subscription::where('expired_at', '<', $timestamp)->orderBy('created_at', 'desc')->paginate(15);
        $title = "إشتراكات المغاسل المنتهية ";
        return view('subscriptions', compact('items', 'title'))->with(['controller' => $this]);
    }

    public function delete(Request $request)
    {
        if (!$request->id) {
            return 'معرف الرسالة غير صالح';
        }
        $item = Subscription::find($request->id);
        if ($item != null) {
            if ($item->delete()) {
                $item->laundry->verification->is_verified = 0;
                $item->laundry->verification->save();
                return "تمت عملية حذف الإشتراك، وإيقاف المغسلة بنجاح";
            } else {
                return "حدث خطأ غير متوقع أثناء الحذف .. حاول في وقت آخر !";
            }
        } else {
            return " تحاول حذف عنصر غير موجود ..  !";
        }
    }

    public function show($id)
    {
        $item = Subscription::find($id);
        if (!$item) {
            return view('errors.404');
        }
        $title = "عرض الإشتراك";
        return view('subscription', compact('item', 'title'))->with(['controller' => $this]);
    }

    public function showUpdate($id)
    {
        $item = Subscription::find($id);
        $plans = SubscriptionPlan::all();
        if (!$item) {
            return view('errors.404');
        }
        $title = "تعديل الإشتراك";
        return view('subscriptionUpdate', compact('item', 'title', 'plans'))->with(['controller' => $this]);
    }

    public function updatePlan(Request $request, $id)
    {
        $plan = SubscriptionPlan::find($request->plan);
        $subscription = Subscription::find($id);
        if (!$plan || !$subscription) {
            return back()->with('success', 'حدث خطأ غير متوقع : أثناء تحديث خطة الإشتراك..');
        }
        $subscription->subscription_plan_id = $plan->id;
        //count expired date, get duration from selected plan
        $timestamp = time();
        //check if current subscription expired
        if ($t = date('Y-m-d H:i:s', $timestamp) > $subscription->expired_at) {
            //expired
            $timestamp += 60 * 60 * 24 * $plan->duration;
            $subscription->expired_at = date('Y-m-d H:i:s', $timestamp);
        } else {
            //not expired
            $expired = strtotime($subscription->expired_at);
            $expired += 60 * 60 * 24 * $plan->duration;
            $subscription->expired_at = date('Y-m-d H:i:s', $expired);
        }
        if (!($subscription->save())) {
            return back()->with('success', 'حدث خطأ غير متوقع : أثناء تحديث خطة الإشتراك..');
        }
        return back()->with('success', 'تم تحديث خطة الإشتراك بنجاح..');
    }

    public function updateDays(Request $request, $id)
    {
        $validator = Validator::make($request->all(), [
            'days' => 'required|numeric',
        ]);
        if ($validator->fails()) {
            return back()->with('success', 'حدث خطأ غير متوقع : أثناء إضافة أيام للإشتراك..');
        }
        $subscription = Subscription::find($id);
        if (!$subscription) {
            return back()->with('success', 'حدث خطأ غير متوقع : أثناء إضافة أيام للإشتراك..');
        }
        //count expired date, get duration from selected plan
        $timestamp = time();
        //check if current subscription expired
        if ($t = date('Y-m-d H:i:s', $timestamp) > $subscription->expired_at) {
            //expired
            $timestamp += 60 * 60 * 24 * $request->days;
            $subscription->expired_at = date('Y-m-d H:i:s', $timestamp);
        } else {
            //not expired
            $expired = strtotime($subscription->expired_at);
            $expired += 60 * 60 * 24 * $request->days;
            $subscription->expired_at = date('Y-m-d H:i:s', $expired);
        }
        if (!($subscription->save())) {
            return back()->with('success', 'حدث خطأ غير متوقع : أثناء إضافة أيام للإشتراك..');
        }
        return back()->with('success', 'تم إضافة أيام للإشتراك بنجاح..');
    }
}
