<?php

namespace App\Http\Controllers\Admin;

use App\Car;
use App\Http\Requests\AdminInserUsertRequest;
use App\Location;
use App\Subscription;
use App\SubscriptionPlan;
use App\User;
use App\UserProfile;
use App\Verification;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class AdminUserController extends Controller
{
    public function index()
    {
        $plans = SubscriptionPlan::all();
        $title = "قائمة المستخدمين ";
        return view('admin.usersList', compact('title', 'plans'))->with(['controller' => $this]);
    }

    public function admins()
    {
        $items = User::where('role', '=', 1)->orderBy('created_at', 'desc')->paginate(15);
        $title = "الإدارة ";
        $users_role = 1;
        return view('admin.users', compact('items', 'title', 'users_role'))->with(['controller' => $this]);
    }

    public function laundries()
    {
        $items = User::where('role', '=', 2)->orderBy('created_at', 'desc')->paginate(15);
        $title = "المغاسل ";
        $users_role = 2;
        return view('admin.users', compact('items', 'title', 'users_role'))->with(['controller' => $this]);
    }

    public function users()
    {
        $items = User::where('role', '=', 3)->orderBy('created_at', 'desc')->paginate(15);
        $title = "المستخدمين ";
        $users_role = 3;
        return view('admin.users', compact('items', 'title', 'users_role'))->with(['controller' => $this]);
    }

    public function store(AdminInserUsertRequest $request)
    {
        //store new user
        $user = new User();
        $user->phone = $request->phone;
        $user->password = bcrypt($request->password);
        $user->role = $request->role;
        if ($user->save()) {
            //verify user
            $verification = new Verification();
            $verification->code = 00000; //for testing
            $verification->is_verified = 1; //for testing
            if (!($user->verification()->save($verification))) {
                $user->delete();
                return back()->with('success', 'حدث خطأ غير متوقع : أثناء عملية التحقق..');
            }
            //store location
            $location = new Location();
            $location->lat = $request->lat;
            $location->long = $request->long;
            if (!($user->location()->save($location))) {
                $user->delete();
                return back()->with('success', 'حدث خطأ غير متوقع : أثناء إضافة  الموقع الجغرافي للمستخدم..');
            }
            //store user profile
            $user_profile = new UserProfile();
            $user_profile->name = $request->name;
            $user_profile->email = $request->email;
            $user_profile->address = $request->address;
            if (!($user->profile()->save($user_profile))) {
                $user->delete();
                return back()->with('success', 'حدث خطأ غير متوقع : أثناء إضافة ملف شخصي للمستخدم..');
            }
            //store laundry subscription to subscriptions
            if ($request->role == 2) {
                $validator = Validator::make($request->all(), [
                    'plan' => 'required|numeric',
                    'company' => 'required|string',
                    'class_model' => 'required|string',
                    'number' => 'required|numeric',
                    'color' => 'required|string',
                    'driver_name' => 'required|string',
                    'driver_phone' => 'required|numeric',
                ]);
                if ($validator->fails()) {
                    $user->delete();
                    return back()->with('success', 'حدث خطأ غير متوقع : أثناء إضافة بيانات السيارة..');
                }

                $subscription = new Subscription();
                //check if plan_id is valid
                $plan = SubscriptionPlan::find($request->plan);
                if (!$plan) {
                    $user->delete();
                    return back()->with('success', 'حدث خطأ غير متوقع : معرف الخطة غير صالح..');
                }
                $subscription->subscription_plan_id = $request->plan;
                //count expired date, get duration from selected plan
                $timestamp = time();
                $timestamp += 60 * 60 * 24 * $plan->duration;
                $subscription->expired_at = date('Y-m-d H:i:s', $timestamp);
                if (!($user->subscription()->save($subscription))) {
                    $user->delete();
                    return back()->with('success', 'حدث خطأ غير متوقع : أثناء إضافة الإشتراك..');
                }
                //store laundry car details
                $car = new Car();
                $car->laundry_id = $user->id;
                $car->company = $request->company;
                $car->class_model = $request->class_model;
                $car->number = $request->number;
                $car->color = $request->color;
                $car->driver_name = $request->driver_name;
                $car->driver_phone = $request->driver_phone;
                if (!($car->save())) {
                    $user->delete();
                    return back()->with('success', 'حدث خطأ غير متوقع : أثناء إضافة تفاصيل السيارة..');
                }

            }
            $user->generateToken();
            return back()->with('success', 'تم عملية الإضافة بنجاح..');
        }
    }

    public function delete(Request $request)
    {
        if (Auth::user()->id == $request->id) {
            return "أنت تحاول حذف نفسك، عملية غير مسموح فيها :(";
        }
        if (!$request->id) {
            return 'معرف المدير غير صالح';
        }
        $user = User::find($request->id);
        if ($user != null) {
            if ($user->delete()) {
                return "تم عملية الحذف بنجاح";
            } else {
                return "حدث خطأ غير متوقع أثناء الحذف .. حاول في وقت آخر !";
            }
        } else {
            return " تحاول حذف عنصر غير موجود ..  !";
        }

    }

    public function marker()
    {
        return view('admin.marker');
    }

    public function show($id)
    {
        $user = User::find($id);
        $plans = SubscriptionPlan::all();
        $title = "بيانات المستخدم";
        return view('admin.user', compact('user', 'title', 'plans'))->with(['controller' => $this]);

    }

    public function map($lat, $lng)
    {

        return view('admin.showUserMap', compact('lat', 'lng'));
    }
}
