<?php

namespace App\Http\Controllers\Admin;

use App\Car;
use App\Http\Requests\AdminInserUsertRequest;
use App\Location;
use App\Subscription;
use App\SubscriptionPlan;
use App\User;
use App\UserProfile;
use App\Verification;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Validator;

class AdminUserController extends Controller
{
    public function index()
    {
        $plans = SubscriptionPlan::all();
        $title = "قائمة المستخدمين ";
        return view('usersList', compact('title', 'plans'))->with(['controller' => $this]);
    }

    public function admins()
    {
        $items = User::where('role', '=', 1)->orderBy('created_at', 'desc')->paginate(15);
        $title = "الإدارة ";
        $users_role = 1;
        return view('users', compact('items', 'title', 'users_role'))->with(['controller' => $this]);
    }

    public function laundries()
    {
        $items = User::where('role', '=', 2)->orderBy('created_at', 'desc')->paginate(15);
        $title = "المغاسل ";
        $users_role = 2;
        return view('users', compact('items', 'title', 'users_role'))->with(['controller' => $this]);
    }

    public function customers()
    {
        $items = User::where('role', '=', 3)->orderBy('created_at', 'desc')->paginate(15);
        $title = "العملاء ";
        $users_role = 3;
        return view('users', compact('items', 'title', 'users_role'))->with(['controller' => $this]);
    }

    public function store(AdminInserUsertRequest $request)
    {
        //store new user
        $user = new User();
        $user->phone = $request->phone;
        $user->password = bcrypt($request->password);
        $user->role = $request->role;
        if ($user->save()) {
            //verify user
            $verification = new Verification();
            $verification->code = 00000; //for testing
            $verification->is_verified = 1; //for testing
            if (!($user->verification()->save($verification))) {
                $user->delete();
                return back()->with('success', 'حدث خطأ غير متوقع : أثناء عملية التحقق..');
            }
            //store location
            $location = new Location();
            $location->lat = $request->lat;
            $location->lng = $request->lng;
            if (!($user->location()->save($location))) {
                $user->delete();
                return back()->with('success', 'حدث خطأ غير متوقع : أثناء إضافة  الموقع الجغرافي للمستخدم..');
            }
            //store user profile
            $user_profile = new UserProfile();
            $user_profile->name = $request->name;
            $user_profile->email = $request->email;
            $user_profile->address = $request->address;
            if (!($user->profile()->save($user_profile))) {
                $user->delete();
                return back()->with('success', 'حدث خطأ غير متوقع : أثناء إضافة ملف شخصي للمستخدم..');
            }
            //store laundry subscription to subscriptions
            if ($request->role == 2) {
                $validator = Validator::make($request->all(), [
                    'plan' => 'required|numeric',
                    'company' => 'required|string',
                    'class_model' => 'required|string',
                    'number' => 'required|string',
                    'color' => 'required|string',
                    'driver_name' => 'required|string',
                    'driver_phone' => 'required|numeric',
                ]);
                if ($validator->fails()) {
                    $user->delete();
                    return back()->with('success', 'حدث خطأ غير متوقع : أثناء إضافة بيانات السيارة..');
                }

                $subscription = new Subscription();
                //check if plan_id is valid
                $plan = SubscriptionPlan::find($request->plan);
                if (!$plan) {
                    $user->delete();
                    return back()->with('success', 'حدث خطأ غير متوقع : معرف الخطة غير صالح..');
                }
                $subscription->subscription_plan_id = $request->plan;
                //count expired date, get duration from selected plan
                $timestamp = time();
                $timestamp += 60 * 60 * 24 * $plan->duration;
                $subscription->expired_at = date('Y-m-d H:i:s', $timestamp);
                if (!($user->subscription()->save($subscription))) {
                    $user->delete();
                    return back()->with('success', 'حدث خطأ غير متوقع : أثناء إضافة الإشتراك..');
                }
                //store laundry car details
                $car = new Car();
                $car->laundry_id = $user->id;
                $car->company = $request->company;
                $car->class_model = $request->class_model;
                $car->number = $request->number;
                $car->color = $request->color;
                $car->driver_name = $request->driver_name;
                $car->driver_phone = $request->driver_phone;
                if (!($car->save())) {
                    $user->delete();
                    return back()->with('success', 'حدث خطأ غير متوقع : أثناء إضافة تفاصيل السيارة..');
                }

            }
            $user->generateToken();
            return back()->with('success', 'تم عملية الإضافة بنجاح..');
        }
    }

    public function delete(Request $request)
    {
        if (Auth::user()->id == $request->id) {
            return "أنت تحاول حذف نفسك، عملية غير مسموح فيها :(";
        }
        if (!$request->id) {
            return 'معرف المدير غير صالح';
        }
        $user = User::find($request->id);
        if ($user != null) {
            /*if($user->profile->photo != null){
                unlink(public_path() . '/media/' . $user->id . '/' . $user->profile->photo);
            }*/
            if ($user->delete()) {
                return "تمت عملية الحذف بنجاح";
            } else {
                return "حدث خطأ غير متوقع أثناء الحذف .. حاول في وقت آخر !";
            }
        } else {
            return " تحاول حذف عنصر غير موجود ..  !";
        }
    }

    public function marker()
    {
        return view('marker');
    }

    public function show($id)
    {
        $user = User::find($id);
        $plans = SubscriptionPlan::all();
        $title = "بيانات المستخدم";
        return view('user', compact('user', 'title', 'plans'))->with(['controller' => $this]);

    }

    public function map($lat, $lng)
    {

        return view('showUserMap', compact('lat', 'lng'));
    }

    /* public function updatePhoto(Request $request, $id){
         try {
             $this->validate($request, [
                 'photo' => 'required|image|mimes:jpeg,jpg,png,bmp|max:8388608',
             ]);
             $user = User::find($id);
             if(!$user){
                 return back()->with('success', 'حدث خلل أثناء تحديث الصورة الشخصية');
             }
             $old_photo = $user->profile->photo;
             $date = date("YMD");
             $new_photo_name = $date.'_'.time() . '_' . $id . '.' . $request->photo->getClientOriginalExtension();
             $user_photo_dir = public_path() . '/media/' . $id;
             if (!File::exists($user_photo_dir)) {
                 File::makeDirectory($user_photo_dir, 0777, true);
             }
             $request->photo->move($user_photo_dir, $new_photo_name);
             $user->profile->photo = $new_photo_name;
             if ($user->profile->save()) {
                 //delete old photos
                 if ($old_photo != null) {
                     unlink(public_path() . '/media/' . $id . '/' . $old_photo);
                 }
                 //end delete old photos
                 return back()->with('success', 'تم تحديث الصورة الشخصية  بنجاح');
             }
         } catch (\Exception $exception) {
             return back()->with('success', 'حدث خلل أثناء تحديث الصورة الشخصية');

         }
     }*/

    public function updateInfo(Request $request, $id)
    {
        $this->validate($request, [
            'name' => 'required|string',
            'phone' => ['required', 'numeric', 'regex:/^(009665|9665|\+9665|05|5)(5|0|3|6|4|9|1|8|7)([0-9]{7})$/'],
            'email' => 'required|max:250|email|string',
            'address' => 'required|max:250|string',
        ]);
        $user = User::find($id);
        if (!$user) {
            return back()->with('success', 'حدث خلل أثناء تحديث البيانات');
        }
        if ($user->phone != $request->input('phone')) {
            $this->validate($request, [
                'phone' => ['unique:users'],
            ]);
        }

        if ($user->profile->email != $request->input('email')) {
            $this->validate($request, [
                'email' => 'unique:users_profiles',
            ]);
        }

        try {
            $user->profile->name = $request->input('name');
            $user->phone = $request->input('phone');
            $user->profile->email = $request->input('email');
            $user->profile->address = $request->input('address');
            if ($user->save() && $user->profile->save()) {
                return back()->with('success', 'تم تحديث البيانات بنجاح');
            } else {
                return back()->with('success', 'حدث خلل أثناء تحديث البيانات');
            }
        } catch (\Exception $exception) {
            return back()->with('success', 'حدث خلل أثناء تحديث البيانات');

        }
    }

    public function updatePassword(Request $request, $id)
    {
        $this->validate($request, [
            'password' => 'required|max:250|string|min:6|confirmed',
        ]);

        try {
            $user = User::find($id);
            if (!$user) {
                return back()->with('success', 'حدث خلل أثناء تحديث كلمة السر');
            }
            $user->password = bcrypt($request->input('password'));
            if ($user->save()) {
                return back()->with('success', 'تم تحديث كلمة السر بنجاح');
            } else {
                return back()->with('success', 'حدث خلل أثناء  تحديث كلمة السر');
            }
        } catch (\Exception $exception) {
            return back()->with('success', 'حدث خلل أثناء تحديث كلمة السر');
        }
    }

    public function updateCar(Request $request, $id)
    {
        $this->validate($request, [
            'company' => 'required|string',
            'class_model' => 'required|string',
            'number' => 'required|string',
            'color' => 'required|string',
            'driver_name' => 'required|string',
            'driver_phone' => 'required|numeric',
        ]);
        $user = User::find($id);
        if (!$user) {
            return back()->with('success', 'حدث خلل أثناء تحديث بيانات السيارة');
        }
        try {
            $car = Car::where('laundry_id', '=', $id)->first();
            if (!$car) {
                return back()->with('success', 'حدث خلل أثناء تحديث بيانات السيارة');
            }
            $car->company = $request->company;
            $car->class_model = $request->class_model;
            $car->number = $request->number;
            $car->color = $request->color;
            $car->driver_name = $request->driver_name;
            $car->driver_phone = $request->driver_phone;
            if ($car->save()) {
                return back()->with('success', 'تم تحديث بيانات السيارة بنجاح');
            } else {
                return back()->with('success', 'حدث خلل أثناء تحديث بيانات السيارة');
            }
        } catch (\Exception $exception) {
            return back()->with('success', 'حدث خلل أثناء تحديث بيانات السيارة');

        }
    }

    public function updateLocation(Request $request, $id)
    {
        $this->validate($request, [
            'lat' => 'required|numeric',
            'lng' => 'required|numeric',
        ]);
        $user = User::find($id);
        if (!$user) {
            return back()->with('success', 'حدث خلل أثناء تحديث الموقع');
        }
        try {
            $user->location->lat = $request->input('lat');
            $user->location->lng = $request->input('lng');
            if ($user->location->save()) {
                return back()->with('success', 'تم تحديث الموقع بنجاح');
            } else {
                return back()->with('success', 'حدث خلل أثناء تحديث الموقع');
            }
        } catch (\Exception $exception) {
            return back()->with('success', 'حدث خلل أثناء تحديث الموقع');

        }
    }

    public function services($id)
    {
        $item = User::find($id);
        $title = "خدمات المغسلة";
        return view('laundryServices', compact('item', 'title'))->with(['controller' => $this]);
    }
}
