<?php

namespace App\Http\Controllers\Api;

use App\AdditionalService;
use App\ClothService;
use App\LaundryAdditionalPrice;
use App\LaundryClothPrice;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class ApiAdditionalServiceController extends Controller
{
    public function items()
    {
        try {
            $user = Auth::guard('api')->user();
            if ($user->role != 2) {
                return response()->json($this->result(false, "هذه الخدمة فقط للمغاسل ", -4));
            }
            if ($this->isSubscriptionExpired($user)) {
                $user->api_token = null;
                $user->save();
                return response()->json($this->result(false, "للأسف الإشتراك الحالي منتهي ولا يمكنك تسجيل الدخول، يرجى تجديد الإشتراك", -6));
            }
            /* $current_clothes_services = LaundryClothPrice::where('laundry_id', '=', $user->id)->get();
             if ($current_clothes_services->count() == 0) {
                 $clothes_items = ClothService::all();
             } else {
                 $clothes_ids = [];
                 //Extract the id's
                 foreach ($current_clothes_services as $q) {
                     array_push($clothes_ids, $q->cloth_service_id);
                 }
                 $clothes_items = ClothService::whereNotIn('id', $clothes_ids)->get();
             }*/
            $current_additional_services = LaundryAdditionalPrice::where('laundry_id', '=', $user->id)->get();
            if ($current_additional_services->count() == 0) {
                $additional_items = AdditionalService::all();
            } else {
                $additional_ids = [];
                //Extract the id's
                foreach ($current_additional_services as $q) {
                    array_push($additional_ids, $q->name);
                }
                $additional_items = AdditionalService::whereNotIn('name', $additional_ids)->get();
            }
            return response()->json(["status" => true, "message" => "تم جلب الخدمات الإضافية الغير مضافة للمغسلة بنجاح", "status_code" => 1, "additional_services_items" => $additional_items]);

            /*            return response()->json($this->resultWithVar(true, "تم جلب الخدمات الإضافية الغير مضافة للمغسلة بنجاح", 1, "additional_services_items", $items));*/
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }

    public function services()
    {
        try {
            $user = Auth::guard('api')->user();
            if ($user->role != 2) {
                return response()->json($this->result(false, "هذه الخدمة فقط للمغاسل ", -4));
            }
            if ($this->isSubscriptionExpired($user)) {
                $user->api_token = null;
                $user->save();
                return response()->json($this->result(false, "للأسف الإشتراك الحالي منتهي ولا يمكنك تسجيل الدخول، يرجى تجديد الإشتراك", -6));
            }
            $items1 = LaundryClothPrice::where('laundry_id', '=', $user->id)->get();
            $items2 = LaundryAdditionalPrice::where('laundry_id', '=', $user->id)->get();
            return response()->json(["status" => true, "message" => "تم جلب خدمات المغسلة بنجاح", "status_code" => 1, "clothes_services" => $items1, "additional_services" => $items2]);
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }

    public function store(Request $request, $id)
    {
        try {
            $user = Auth::guard('api')->user();
            if ($user->role != 2) {
                return response()->json($this->result(false, "هذه الخدمة فقط للمغاسل ", -4));
            }
            if ($this->isSubscriptionExpired($user)) {
                $user->api_token = null;
                $user->save();
                return response()->json($this->result(false, "للأسف الإشتراك الحالي منتهي ولا يمكنك تسجيل الدخول، يرجى تجديد الإشتراك", -6));
            }
            //request validation
            $validator = Validator::make($request->all(), [
                'washing' => ['required', 'numeric'],
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
            }
            //check if id belong to additional service
            $additional_service = AdditionalService::find($id);
            if (!$additional_service) {
                return response()->json($this->result(false, "تعذر الحصول على الخدمة الإضافية المراد إضافتها", -3));
            }
            //check if additional service price in current laundry additional services
            $additional_service_price = LaundryAdditionalPrice::where('laundry_id', '=', $user->id)->where('additional_service_id', '=', $id)->first();
            if (!$additional_service_price) {
                //store
                $item = new LaundryAdditionalPrice();
                $item->additional_service_id = $id;
                $item->washing = $request->washing;
                if ($user->laundryAdditionalPrices()->save($item)) {
                    $inserted_item = LaundryAdditionalPrice::with('additionalService')->find($item->id);
                    return response()->json($this->resultWithVar(true, "تم إضافة خدمة إضافية جديدة للمغسلة بنجاح", 1, "additional_service_price", $inserted_item));
                } else {
                    return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
                }
            } else {
                //update
                $additional_service_price->washing = $request->washing;
                if ($additional_service_price->save()) {
                    $updated_additional_service_price = LaundryAdditionalPrice::with('additionalService')->find($additional_service_price);
                    return response()->json($this->resultWithVar(true, "تم تحديث أسعار الخدمة بنجاح", 3, "additional_service_price", $updated_additional_service_price));
                } else {
                    return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
                }
            }
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }

    /*    public function show()
        {
            try {
                $user = Auth::guard('api')->user();
                if ($user->role != 2) {
                    return response()->json($this->result(false, "هذه الخدمة فقط للمغاسل ", -4));
                }
                $items = LaundryAdditionalPrice::with('additionalService')->where('laundry_id', '=', $user->id)->get();
                return response()->json($this->resultWithVar(true, "تم جلب خدمات الملابس للمغسلة بنجاح", 1, "additional_services", $items));
            } catch (\Exception $exception) {
                return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
            }
        }*/

    public function delete($id)
    {
        try {
            $user = Auth::guard('api')->user();
            if ($user->role != 2) {
                return response()->json($this->result(false, "هذه الخدمة فقط للمغاسل ", -4));
            }
            if ($this->isSubscriptionExpired($user)) {
                $user->api_token = null;
                $user->save();
                return response()->json($this->result(false, "للأسف الإشتراك الحالي منتهي ولا يمكنك تسجيل الدخول، يرجى تجديد الإشتراك", -6));
            }
            $item = LaundryAdditionalPrice::where('id', '=', $id)->where('laundry_id', '=', $user->id)->first();
            if (!$item) {
                return response()->json($this->result(false, "تعذر الحصول على الخدمة الإضافية المراد حذفها", -3));
            }
            if (!($item->delete())) {
                return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
            }
            return response()->json($this->result(true, "تم حذف الخدمة الإضافية بنجاح", 1));
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }
}
