<?php

namespace App\Http\Controllers\Api;

use App\Rating;
use App\User;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class ApiRatingController extends Controller
{
    public function store(Request $request, $laundry_id)
    {
        try {
            $user = Auth::guard('api')->user();
            if ($user->role != 3) {
                return response()->json($this->result(false, "هذه الخدمة فقط للعملاء ", -4));
            }
            $validator = Validator::make($request->all(), [
                'rate' => ['numeric', 'required', 'regex:/[1-5]/'],
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
            }
            //check if laundry is found
            $laundry = User::where('id', '=', $laundry_id)->where('role', '=', 2)->first();
            if (!$laundry) {
                return response()->json($this->result(false, "تعذر الحصول على المغسلة المراد تقييمها", -3));
            }
            //check if rate is found
            $rate = Rating::where('user_id', '=', $user->id)->where('laundry_id', '=', $laundry_id)->first();
            if ($rate) {
                //update rate
                $rate->rate = $request->rate;
                if (!$rate->save()) {
                    return response()->json($this->result(false, "حدث خطأ غير متوقع أثناء تحديث تقييم المغسلة", -2));
                }
                $returned_rate = Rating::with('laundry', 'laundry.profile')->find($rate->id);
                return response()->json($this->resultWithVar(true, "تم تحديث تقييم المغسلة بنجاح", 2, "rate", $returned_rate));
            }
            //create new rate
            $rate = new Rating();
            $rate->user_id = $user->id;
            $rate->laundry_id = $laundry_id;
            $rate->rate = $request->rate;
            if ($rate->save()) {
                $returned_rate = Rating::with('laundry', 'laundry.profile')->find($rate->id);
                return response()->json($this->resultWithVar(true, "تم تقييم المغسلة بنجاح", 1, "rate", $returned_rate));
            } else {
                return response()->json($this->result(false, "حدث خطأ غير متوقع أثناء تقييم المغسلة", -2));
            }
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خطأ غير متوقع أثناء إرسال البيانات، حاول في وقت آخر", 1));

        }
    }

    public function delete($laundry_id)
    {
        try {
            $user = Auth::guard('api')->user();
            if ($user->role != 3) {
                return response()->json($this->result(false, "هذه الخدمة فقط للعملاء ", -4));
            }
            $item = Rating::where('user_id', '=', $user->id)->where('laundry_id', '=', $laundry_id)->first();
            if (!$item) {
                return response()->json($this->result(false, "تعذر الحصول على التقييم المراد حذفه", -3));
            }
            if (!($item->delete())) {
                return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
            }
            return response()->json($this->result(true, "تم حذف التقييم بنجاح", 1));
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }
}

