<?php

namespace App\Http\Controllers\Api;

use App\Car;
use App\Location;
use App\SubscriptionOrder;
use App\SubscriptionPlan;
use App\User;
use App\UserProfile;
use App\UserWithToken;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Foundation\Auth\AuthenticatesUsers;


class ApiUserController extends Controller
{
    use AuthenticatesUsers;

    public function store(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'name' => ['required', 'string'],
                'phone' => ['required', 'numeric', 'unique:users', 'regex:/^(009665|9665|\+9665|05|5)(5|0|3|6|4|9|1|8|7)([0-9]{7})$/'],
                'password' => 'required|max:250|string|min:6|confirmed',
                'email' => 'required|max:250|email|string|unique:users_profiles',
                'address' => 'required|max:250|string',
                'role' => ['numeric', 'required', 'regex:/[2-3]/'],
                'lat' => 'required|numeric',
                'lng' => 'required|numeric',
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
            }
            //store new user
            $user = new User();
            $user->phone = $request->phone;
            $user->password = bcrypt($request->password);
            $user->role = $request->role;
            if ($user->save()) {
                //store location
                $location = new Location();
                $location->lat = $request->lat;
                $location->lng = $request->lng;
                if (!($user->location()->save($location))) {
                    $user->delete();
                    return response()->json($this->result(false, "حدث خطأ غير متوقع : أثناء إضافة  الموقع الجغرافي للمستخدم", -2));
                }
                //store user profile
                $user_profile = new UserProfile();
                $user_profile->name = $request->name;
                $user_profile->email = $request->email;
                $user_profile->address = $request->address;
                if (!($user->profile()->save($user_profile))) {
                    $user->delete();
                    return response()->json($this->result(false, "حدث خطأ غير متوقع أثناء عملية التسجيل", -2));
                }
                //store laundry subscription to subscriptions
                if ($request->role == 2) {
                    $validator = Validator::make($request->all(), [
                        'plan_id' => 'required|numeric',
                        'company' => 'required|string',
                        'class_model' => 'required|string',
                        'number' => 'required|string',
                        'color' => 'required|string',
                        'driver_name' => ['required', 'string'],
                        'driver_phone' => ['required', 'numeric', 'regex:/^(009665|9665|\+9665|05|5)(5|0|3|6|4|9|1|8|7)([0-9]{7})$/'],
                    ]);
                    if ($validator->fails()) {
                        $user->delete();
                        return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
                    }
                    //check if plan_id is valid
                    $plan = SubscriptionPlan::find($request->plan_id);
                    if (!$plan) {
                        $user->delete();
                        return response()->json($this->result(false, "حدث خطأ غير متوقع :تأكد من اختبار الخطة بالشكل الصحيح", -3));
                    }
                    $subscription_order = new SubscriptionOrder();
                    $subscription_order->laundry_id = $user->id;
                    $subscription_order->subscription_plan_id = $plan->id;
                    if (!($subscription_order->save())) {
                        $user->delete();
                        return response()->json($this->result(false, "حدث خطأ غير متوقع : أثناء إضافة طلب الإشتراك", -3));
                    }
                    //store laundry car details
                    $car = new Car();
                    $car->laundry_id = $user->id;
                    $car->company = $request->company;
                    $car->class_model = $request->class_model;
                    $car->number = $request->number;
                    $car->color = $request->color;
                    $car->driver_name = $request->driver_name;
                    $car->driver_phone = $request->driver_phone;
                    if (!($car->save())) {
                        $user->delete();
                        return response()->json($this->result(false, "حدث خطأ غير متوقع : أثناء إضافة تفاصيل السيارة", -3));
                    }
                }
                return $this->registered($request, $user)
                    ?: response()->json($this->result(false, "حدث خطأ غير متوقع : أثناء عملية التسجيل", -2));

            } else {
                return response()->json($this->result(false, "حدث خطأ غير متوقع : أثناء عملية التسجيل", -2));
            }
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خطأ غير متوقع : أثناء عملية التسجيل", -2));
        }
    }

    /**
     * The user has been registered.
     *
     * @param  \Illuminate\Http\Request $request
     * @param  mixed $user
     * @return mixed
     */

    protected function registered(Request $request, $user)
    {
        $user->generateActivation($user);
        if ($user->role == 3) {
            return response()->json($this->result(true, "تمت عملية تسجيلك بنجاح، تم إرسال رسالة تأكيد لرقم جوالك، أدخل الرمز لإنهاء عملية التسجيل", 1));
        }
        return response()->json($this->result(true, "تمت عملية تسجيلك بنجاح، سيتم تأكيد حسابك من قبل إدارة التطبيق لإنهاء عملية التسجيل", 1));
    }

    public function verify(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'phone' => ['required', 'regex:/^(009665|9665|\+9665|05|5)(5|0|3|6|4|9|1|8|7)([0-9]{7})$/'],
                'code' => 'required|numeric',
                'password' => 'required|string',
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
            }
            //check if user found with request phone
            $user = User::where('phone', '=', $request->phone)->first();
            if ($user == null) {
                return response()->json($this->result(false, "تعذر الحصول على المستخدم", -3));
            }
            //check if user is a customer
            if ($user->role != 3) {
                return response()->json($this->result(false, "خدمة التحقق متوفرة فقط للزبائن ", -4));
            }
            //check if user verified before
            if ($user->verification->is_verified == 1) {
                return response()->json($this->result(false, "تم التحقق من الحساب مسبقاً ", 2));
            }
            //check if code is valid
            if ($user->verification->code == $request->code and Hash::check($request->password, $user->password)) {
                $user->verification->is_verified = 1;
                $user->verification->save();
                $user->generateToken();
                $returned_user = UserWithToken::with('profile', 'location')->find($user->id);
                $this->guard()->user();
                return response()->json($this->resultWithVar(true, "تم التحقق من الحساب بنجاح", 1, "user", $returned_user));
            } else {
                return response()->json($this->result(false, "البيانات غير متطابقة، تأكد من الرمز مرة أخرى ", -1));
            }
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء عملية تأكيد الرمز، حاول في وقت آخر ", -2));
        }
    }

    /* protected function guard()
     {
         return Auth::guard();
     }*/

    public function logout(Request $request)
    {
        $user = Auth::guard('api')->user();
        if ($user) {
            $user->api_token = null;
            $user->save();
        }
        return response()->json($this->result(true, "تم تسجيل الخروج بنجاح", 1));
    }

    /*protected function attemptLogin(Request $request)
    {
        return $this->guard()->attempt(
            $this->credentials($request), $request->has('remember')
        );
    }*/

    public function forgetPassword(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'phone' => ['required', 'regex:/^(009665|9665|\+9665|05|5)(5|0|3|6|4|9|1|8|7)([0-9]{7})$/'],
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
            }
            //check if user found with request phone
            $user = User::where('phone', '=', $request->phone)->first();
            if ($user == null) {
                return response()->json($this->result(false, "لا يوجد أي حساب يطابق رقم الهاتف الذي أدخلته", -3));
            }
            $new_password = $this->randomPassword();
            /*            $user->password = Hash::make($new_password);*/
            $user->password = Hash::make('b9xJMr4c');
            if ($user->save()) {
                if ($this->sendSMS($user->phone, " تم إسترداد كلمة المرور بناءا على طلب، كلمة المرور الجديدة هي\n" . $new_password . "\nتطبيق مغاسل")) {
                    return response()->json($this->result(true, "تمت العملية بنجاح، تم إرسال كلمة المرور الجديدة إلى هاتفك", 1));
                } else {
                    return response()->json($this->result(false, "حدث خطأ غير متوقع أثناء إسترداد كلمة المرور", -2));
                }
            } else {
                return response()->json($this->result(false, "حدث خطأ غير متوقع أثناء إسترداد كلمة المرور", -2));
            }
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خطأ غير متوقع أثناء إسترداد كلمة المرور", -2));
        }
    }

    public function randomPassword()
    {
        $alphabet = "abcdefghijklmnopqrstuwxyzABCDEFGHIJKLMNOPQRSTUWXYZ0123456789";
        $pass = array(); //remember to declare $pass as an array
        $alphaLength = strlen($alphabet) - 1; //put the length -1 in cache
        for ($i = 0; $i < 8; $i++) {
            $n = rand(0, $alphaLength);
            $pass[] = $alphabet[$n];
        }
        return implode($pass); //turn the array into a string
    }

    public function checkSubscription()
    {
        try {
            $user = Auth::guard('api')->user();
            if ($user->role != 2) {
                return response()->json($this->result(false, "هذه الخدمة فقط للمغاسل ", -4));
            }
            if ($this->isSubscriptionExpired($user)) {
                $user->api_token = null;
                $user->save();
                return response()->json($this->result(false, "للأسف الإشتراك الحالي منتهي ولا يمكنك تسجيل الدخول، يرجى تجديد الإشتراك", -1));
            }
            return response()->json(["status" => true, "message" => "الإشتراك الحالي فعال", "status_code" => 1]);

            /*            return response()->json($this->resultWithVar(true, "تم جلب الخدمات الإضافية الغير مضافة للمغسلة بنجاح", 1, "additional_services_items", $items));*/
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }

    public function laundryProfile($laundry_id)
    {
        try {
            //detect user
            $user = Auth::guard('api')->user();
            //check if user is a customer
            if ($user->role != 3) {
                return response()->json($this->result(false, "هذه الخدمة فقط للعملاء ", -4));
            }
            $timestamp = time();
            $item = DB::table('users')
                ->join('users_profiles', 'users_profiles.user_id', '=', 'users.id')
                ->join('subscriptions', 'subscriptions.laundry_id', '=', 'users.id')
                ->where('subscriptions.expired_at', '>', date('Y-m-d H:i:s', $timestamp))
                ->where('users.id', '=', $laundry_id)
                ->where('users.role', '=', 2)
                ->select(DB::raw('users.id'))
                ->take(1)
                ->get();
            if ($item->count() == 0) {
                return response()->json($this->result(false, "تعذر الحصول على المغسلة", -3));
            }
            $ids = [];
            //Extract the id's
            foreach ($item as $i) {
                array_push($ids, $i->id);
            }
            $customer_id = $user->id;
            $laundry = User::with(['profile', 'laundryClothesPrices', 'laundryAdditionalPrices', 'has_rate' => function ($q) use ($customer_id/*, $laundry_id*/) {
                $q->where('user_id', '=', $customer_id)/*->where('laundry_id', '=', $laundry_id)*/
                ;
            }])->whereIn('id', $ids)->first();
            if (!$laundry) {
                return response()->json($this->result(false, "تعذر الحصول على المغسلة", -3));
            }

            return response()->json(["status" => true, "message" => "تم جلب ملف المغسلة بنجاح", "status_code" => 1, "laundry" => $laundry, "laundry_rate" => floor($this->getRate($laundry->id))]);
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }

    public function profile(Request $request)
    {
        try {
            //detect user
            $user = Auth::guard('api')->user();
            //check if user is a customer
            if ($user->role != 2) {
                return response()->json($this->result(false, "هذه الخدمة فقط للمغاسل ", -4));
            }
            /*$timestamp = time();
            $item = DB::table('users')
                ->join('users_profiles', 'users_profiles.user_id', '=', 'users.id')
                ->join('subscriptions', 'subscriptions.laundry_id', '=', 'users.id')
                ->where('subscriptions.expired_at', '>', date('Y-m-d H:i:s', $timestamp))
                ->where('users.id', '=', $laundry_id)
                ->where('users.role', '=', 2)
                ->select(DB::raw('users.id'))
                ->take(1)
                ->get();
            if ($item->count() == 0) {
                return response()->json($this->result(false, "تعذر الحصول على المغسلة", -3));
            }
            $ids = [];
            //Extract the id's
            foreach ($item as $i) {
                array_push($ids, $i->id);
            }*/
            $laundry = User::with(['profile', 'location', 'car', 'subscription'])->find($user->id);
            return response()->json(["status" => true, "message" => "تم جلب الملف الشخصي المغسلة بنجاح", "status_code" => 1, "laundry" => $laundry, "laundry_rate" => floor($this->getRate($laundry->id))]);
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }

    public function update(Request $request)
    {
        /*        try {*/
        $user = Auth::guard('api')->user();
        //check input if sent
        if ($request->has('password')) {
            //validate
            $validator = Validator::make($request->all(), [
                'password' => ['required', 'confirmed', 'max:250', 'string', 'min:6'],
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
            }
            //if validation passed then update this field
            $user->password = Hash::make($request->password);
        }

        //check input if sent
        if ($request->has('name')) {
            //validate
            $validator = Validator::make($request->all(), [
                'name' => ['required', 'string'],
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
            }
            //if validation passed then update this field
            $user->profile->name = $request->name;
            $user->profile->save();
        }


        //check input if sent
        if ($request->has('email')) {
            //validate
            if ($user->profile->email == $request->email) {
                $validator = Validator::make($request->all(), [
                    'email' => ['required', 'max:250', 'email', 'string'],
                ]);
                if ($validator->fails()) {
                    return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
                }
            } else {
                $validator = Validator::make($request->all(), [
                    'email' => ['required', 'max:250', 'email', 'string', 'unique:users_profiles'],
                ]);
                if ($validator->fails()) {
                    return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
                }
            }
            //if validation passed then update this field
            $user->profile->email = $request->email;
        }

        //check input if sent
        if ($request->has('address')) {
            //validate
            $validator = Validator::make($request->all(), [
                'address' => ['required', 'max:250', 'string'],
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
            }

//if validation passed then update this field
            $user->profile->address = $request->address;
        }

//check input if sent
        if ($request->has('lat')) {
            //validate
            $validator = Validator::make($request->all(), [
                'lat' => ['required', 'numeric'],
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
            }
            //if validation passed then update this field
            $user->location->lat = $request->lat;
        }

//check input if sent
        if ($request->has('lng')) {
            //validate
            $validator = Validator::make($request->all(), [
                'lng' => ['required', 'numeric'],
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
            }
            //if validation passed then update this field
            $user->location->lng = $request->lng;
        }

//update laundry car details
        if ($user->role == 2) {

            //check input if sent
            if ($request->has('company')) {
                //validate
                $validator = Validator::make($request->all(), [
                    'company' => ['required', 'string'],
                ]);
                if ($validator->fails()) {
                    return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
                }
                //if validation passed then update this field
                $user->car->company = $request->company;
            }

            //check input if sent
            if ($request->has('class_model')) {
                //validate
                $validator = Validator::make($request->all(), [
                    'class_model' => ['required', 'string'],
                ]);
                if ($validator->fails()) {
                    return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
                }
                //if validation passed then update this field
                $user->car->class_model = $request->class_model;
            }

            //check input if sent
            if ($request->has('number')) {
                //validate
                $validator = Validator::make($request->all(), [
                    'number' => ['required', 'string'],
                ]);
                if ($validator->fails()) {
                    return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
                }
                //if validation passed then update this field
                $user->car->number = $request->number;
            }

            //check input if sent
            if ($request->has('color')) {
                //validate
                $validator = Validator::make($request->all(), [
                    'color' => ['required', 'string'],
                ]);
                if ($validator->fails()) {
                    return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
                }
                //if validation passed then update this field
                $user->car->color = $request->color;
            }

            //check input if sent
            if ($request->has('driver_name')) {
                //validate
                $validator = Validator::make($request->all(), [
                    'driver_name' => ['required', 'string'],
                ]);
                if ($validator->fails()) {
                    return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
                }
                //if validation passed then update this field
                $user->car->driver_name = $request->driver_name;
            }

            //check input if sent
            if ($request->has('driver_phone')) {
                //validate
                $validator = Validator::make($request->all(), [
                    'driver_phone' => ['required', 'numeric', 'regex:/^(009665|9665|\+9665|05|5)(5|0|3|6|4|9|1|8|7)([0-9]{7})$/'],
                ]);
                if ($validator->fails()) {
                    return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
                }
                //if validation passed then update this field
                $user->car->driver_phone = $request->driver_phone;
            }

            $user->car->save();
        }
//end update car

        $user->save();
        $user->profile->save();

//return updated user if laundry return also car details
        if ($user->role == 2) {
            $updated_user = User::with('profile', 'location', 'car')->find($user->id);
            return response()->json($this->resultWithVar(true, "تم تحديث البيانات الشخصية بنجاح", 2, "user", $updated_user));
        }

//if user is a customer
        $updated_user = User::with('profile', 'location')->find($user->id);
        return response()->json($this->resultWithVar(true, "تم تحديث البيانات الشخصية بنجاح", 1, "user", $updated_user));
        /*  } catch (\Exception $exception) {
              return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
          }*/
    }

    function updateStatus(Request $request)
    {
        try {
            $user = Auth::guard('api')->user();
            if ($user->role != 2) {
                return response()->json($this->result(false, "هذه الخدمة فقط للمغاسل ", -4));
            }

            /* if ($this->isSubscriptionExpired($user)) {
                 $user->api_token = null;
                 $user->save();
                 return response()->json($this->result(false, "للأسف الإشتراك الحالي منتهي ولا يمكنك تسجيل الدخول، يرجى تجديد الإشتراك", -1));
             }*/

            $validator = Validator::make($request->all(), [
                'isActive' => ['boolean', 'required'],
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
            }
            $user->profile->isActive = $request->isActive;
            if ($user->profile->save()) {
                $updated_user = User::with('profile', 'location', 'car')->find($user->id);
                return response()->json($this->resultWithVar(true, "تم تحديث الحالة بنجاح", 1, "laundry", $updated_user));
            } else {
                return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
            }
            /*            return response()->json($this->resultWithVar(true, "تم جلب الخدمات الإضافية الغير مضافة للمغسلة بنجاح", 1, "additional_services_items", $items));*/
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }

    function test(Request $request)
    {

        $new_password = $this->randomPassword();
        return " تم إسترداد كلمة المرور بناءا على طلب، كلمة المرور الجديدة هي\n" . $new_password . "\nتطبيق مغاسل";
    }
}
