<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\User;
use App\UserWithToken;
use Illuminate\Foundation\Auth\AuthenticatesUsers;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Request;
use Illuminate\Support\Facades\Validator;

class LoginController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles authenticating users for the application and
    | redirecting them to your home screen. The controller uses a trait
    | to conveniently provide its functionality to your applications.
    |
    */

    use AuthenticatesUsers;

    /**
     * Where to redirect users after login.
     *
     * @var string
     */
    protected $redirectTo = '/home';

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest')->except('logout');
    }

    public function apiLogin(Request $request)
    {
        //check session
        if (Auth::check()) {
            return response()->json($this->result(false, "حدث خلل غير متوقع، أنت قيد تسجيل الدخول، تم تسجيل دخولك مسبقاً", 0));
        }
        //validation
        $validator = Validator::make($request->all(), [
            'phone' => ['required', 'numeric', 'regex:/^(009665|9665|\+9665|05|5)(5|0|3|6|4|9|1|8|7)([0-9]{7})$/'],
            'password' => 'required|max:250|string',
        ]);
        if ($validator->fails()) {
            return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $validator->errors()));
        }
        //check password
        if(! (Auth::attempt(array('phone' => $request->phone, 'password' => $request->password)))){
            return response()->json($this->result(false, "البيانات المدخلة غير صحيحة", -1));
        }
        /* $user = User::where('phone', '=', $request->phone)->where('password','=', bcrypt($request->password))->first();
         if(!$user){
         }*/

        //check verification
        //for laundry
        $user = User::where('phone', '=', $request->phone)->first();
        if($user->verification->is_verified != 1 and $user->role == 2){
            return response()->json($this->result(false, "لم يتم التحقق من الحساب، بإنتظار قبول طلب الإشتراك من قبل الإدارة", -3));
        }
        //for customer
        if($user->verification->is_verified != 1 and $user->role == 3){
            return response()->json($this->result(false, "لم يتم التحقق من الحساب، لم يتم ادخال رمز التحقق بالشكل الصحيح", -3));
        }
        //for admin
        if($user->verification->is_verified != 1){
            return response()->json($this->result(false, "لم يتم التحقق من الحساب", -3));
        }
        $this->validateLogin($request);
        // If the class is using the ThrottlesLogins trait, we can automatically throttle
        // the login attempts for this application. We'll key this by the username and
        // the IP address of the client making these requests into this application.
        if ($this->hasTooManyLoginAttempts($request)) {
            $this->fireLockoutEvent($request);
            return $this->sendLockoutResponse($request);
        }
        if ($this->attemptLogin($request)) {
            $user = $this->guard()->user();
            $user->generateToken();
            $user_with_token = UserWithToken::with('profile','location')->find($user->id);
            return response()->json($this->resultWithVar(true,"تم تسجيل دخولك بنجاح", 1, "user", $user_with_token));
        }
        // If the login attempt was unsuccessful we will increment the number of attempts
        // to login and redirect the user back to the login form. Of course, when this
        // user surpasses their maximum number of attempts they will get locked out.
        $this->incrementLoginAttempts($request);
        return $this->sendFailedLoginResponse($request);
        //start api_token session
/*        $user = $this->guard()->user();*/
        //return user information depends on role
        /*if($user->role == 2){
            $laundry_with_token = UserWithToken::find($user->id);
            return response()->json($this->resultWithVar(true,"تم تسجيل دخولك بنجاح", 1, "user", $laundry_with_token));
        }else{*/
       /* $user_with_token = UserWithToken::with('profile','location')->find($user->id);
        return response()->json($this->resultWithVar(true,"تم تسجيل دخولك بنجاح", 1, "user", $user_with_token));*/
        /*        }*/
    }

}
