<?php

namespace App\Http\Controllers;

use App\LaundryAdditionalPrice;
use App\LaundryClothPrice;
use App\Message;
use App\Order;
use App\Rating;
use App\Subscription;
use App\SubscriptionOrder;
use App\User;
use App\UserProfile;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Illuminate\Routing\Controller as BaseController;
use Illuminate\Foundation\Validation\ValidatesRequests;
use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use GuzzleHttp\Client;

class Controller extends BaseController
{
    use AuthorizesRequests, DispatchesJobs, ValidatesRequests;

    //json responses functions
    public function result($status, $message, $status_code)
    {
        $arr = array('status' => $status, 'message' => $message, 'status_code' => $status_code);
        return $arr;
    }

    public function resultWithVar($status, $message, $status_code, $result_name, $result_value)
    {
        $arr = array("status" => $status, "message" => $message, "status_code" => $status_code, $result_name => $result_value);
        return $arr;
    }

    public function validatorErrorsToStringArray($validator)
    {
        $array = [];
        foreach ($validator->errors()->messages() as $key => $value) {
            foreach ($value as $message) {
                array_push($array, $message);
            }
        }
        return $array;
    }

    public function isSubscriptionExpired(User $user)
    {
        $timestamp = time();
        if ($user->subscription->expired_at < date('Y-m-d H:i:s', $timestamp)) {
            return true;
        } else {
            return false;
        }
    }

    public function getLaundryName($laundry_id)
    {
        $user_profile = UserProfile::where('user_id', '=', $laundry_id)->first();
        if (!$user_profile) {
            return "unknown";
        } else {
            return $user_profile->name;
        }

    }

    public function getUserByToken($auth)
    {
        $request_token = substr($auth, strpos($auth, " ") + 1);
        $user = User::where('api_token', '=', $request_token)->first();
        return $user;
    }

    public function sendSMS($phone, $text)
    {
        try {
            $client = new \GuzzleHttp\Client(); //GuzzleHttp\Client

            $username = "966560705959";
            $password = "12345678";
            $number = $phone;
            /*        $sender = "Mghasel";*/
            $sender = "HISMS.WS-AD";
            $message = $text;

            $headers = ['Content-type' => 'text/plain; charset=utf-8'];

            $client->get("https://www.hisms.ws/api.php?send_sms&username=$username&password=$password&numbers=966$number&sender=$sender&message=$message", ['headers' => $headers]);
            return true;
        } catch (\Exception $exception) {
            return false;
        }
    }

    public function getRate($laundry_id)
    {
        $ratings = Rating::where('laundry_id', '=', $laundry_id)->get();
        if ($ratings->count() == null) {
            return null;
        }
        $total_ratings = 0;
        foreach ($ratings as $rating) {
            $total_ratings += $rating->rate;
        }
        return $total_ratings / $ratings->count();
    }

    /*    public function validationErrorsToString($errArray) {
            $valArr = array();
            foreach ($errArray->toArray() as $key => $value) {
                $errStr = $key.' '.$value[0];
                array_push($valArr, $errStr);
            }
            if(!empty($valArr)){
                $errStrFinal = implode(',', $valArr);
            }
            return $errStrFinal;
        }*/

    /*public function result($status, $message, $status_code){
        $arr = array('status' => $status, 'message' => $message, 'status_code' => $status_code);
        return $arr;
    }*/

    function time_ago($date)
    {
        if (empty($date)) {
            return "الوقت غير محدد";
        }

        $periods = array("ثانية", "دقيقة", "ساعة", "يوم", "أسبوع", "شهر", "سنة", "decade");
        $lengths = array("60", "60", "24", "7", "4.35", "12", "10");

        $now = time();
        $unix_date = strtotime($date);

        // check validity of date
        if (empty($unix_date)) {
            return "صيغة غير صالحة للتاريخ";
        }

        // is it future date or past date
        if ($now > $unix_date) {
            $difference = $now - $unix_date;
            $tense = "منذ";

        } else {
            $difference = $unix_date - $now;
            $tense = "من الآن";
        }

        for ($j = 0; $difference >= $lengths[$j] && $j < count($lengths) - 1; $j++) {
            $difference /= $lengths[$j];
        }

        $difference = round($difference);

        if ($difference != 1) {
            $periods[$j] .= "";
        }

        return "{$tense} $difference $periods[$j]";
    }

    function remained_days($expired_at)
    {
        $timestamp = time();
        $t = date('Y-m-d H:i:s', $timestamp);
        $now = date_create($t);
        $expired = date_create($expired_at);
        $interval = $now->diff($expired);
        return $interval->format('%R%a يوم');
    }

    public function getClothWhiteCost($laundry_cloth_price_id, $washingOrIroning, $is_colored = 0)
    {
        //get laundry cloth price
        $laundry_cloth_price = LaundryClothPrice::find($laundry_cloth_price_id);
        if (!$laundry_cloth_price) {
            return 0;
        }
        //calculate washing price
        //get washing price

        if ($washingOrIroning == 0) {
            //just washing
            if ($is_colored == 0) {
                //white
                return $laundry_cloth_price->white_washing;
            } elseif ($is_colored == 1) {
                //it colored
                return $laundry_cloth_price->colored_washing;
            }
        } elseif ($washingOrIroning == 1) {
            //it just ironing
            return $laundry_cloth_price->ironing;
        } elseif ($washingOrIroning == 2) {
            //it both
            if ($is_colored == 0) {
                //it white
                return $laundry_cloth_price->white_washing + $laundry_cloth_price->ironing;
            } elseif ($is_colored == 1) {
                //it colored
                return $laundry_cloth_price->colored_washing + $laundry_cloth_price->ironing;
            }
        } else {
            return 0;
        }

    }

    public function getClothColoredCost($laundry_cloth_price_id, $washingOrIroning, $is_colored = 1)
    {
        //get laundry cloth price
        $laundry_cloth_price = LaundryClothPrice::find($laundry_cloth_price_id);
        if (!$laundry_cloth_price) {
            return 0;
        }
        //calculate washing price
        //get washing price

        if ($washingOrIroning == 0) {
            //just washing
            if ($is_colored == 0) {
                //white
                return $laundry_cloth_price->white_washing;
            } elseif ($is_colored == 1) {
                //it colored
                return $laundry_cloth_price->colored_washing;
            }
        } elseif ($washingOrIroning == 1) {
            //it just ironing
            return $laundry_cloth_price->ironing;
        } elseif ($washingOrIroning == 2) {
            //it both
            if ($is_colored == 0) {
                //it white
                return $laundry_cloth_price->white_washing + $laundry_cloth_price->ironing;
            } elseif ($is_colored == 1) {
                //it colored
                return $laundry_cloth_price->colored_washing + $laundry_cloth_price->ironing;
            }
        } else {
            return 0;
        }

    }

    /* public function getAdditionalCost($laundry_additional_price_id)
     {
         //get laundry additional service price
         $laundry_additional_price = LaundryAdditionalPrice::find($laundry_additional_price_id);
         if (!$laundry_additional_price) {
             return 0;
         }
          return $laundry_additional_price->washing;
     }*/

    /* function estateFirstFile($id){
         $file = File::where('estate_id','=', $id)->first();
         if($file != null)
         return $file->name;
         else
             return 0;
     }*/

    /*counters*/

    /*counting*/
    public function messagesCount()
    {
        return Message::all()->count();
    }

    public function unreadMessagesCount()
    {
        $unread_messages = Message::where('is_read', '=', 0);
        return $unread_messages->count();
    }

    public function usersCount()
    {
        try {
            return User::all()->count();
        } catch (\Exception $exception) {
            return 0;
        }
    }

    public function subscriptionOrdersCount()
    {
        try {
            return SubscriptionOrder::where('is_accepted', '=', -1)->count();
        } catch (\Exception $exception) {
            return 0;
        }
    }

    public function subscriptionsCount()
    {
        try {
            return Subscription::all()->count();
        } catch (\Exception $exception) {
            return 0;
        }
    }

    public function ordersCount()
    {
        try {
            return Order::all()->count();
        } catch (\Exception $exception) {
            return 0;
        }
    }

    //skip apple security
    public function skipBtnFun(){
        return response()->json(true);
    }
    /*counting*/
}
