<?php

namespace App;

use Illuminate\Notifications\Notifiable;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Support\Facades\DB;
use GuzzleHttp\Client;

class User extends Authenticatable
{

    use Notifiable;

    protected $table = "users";

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'phone', 'password', 'role'
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token', 'api_token'
    ];

    public function profile()
    {
        return $this->hasOne(UserProfile::class);
    }

    public function verification()
    {
        return $this->hasOne(Verification::class);
    }

    public function location()
    {
        return $this->hasOne(Location::class);
    }

    public function subscription()
    {
        return $this->hasOne(Subscription::class, 'laundry_id');
    }

    public function receivedOrders()
    {
        return $this->hasMany(Order::class, 'laundry_id');
    }

    public function SentOrders()
    {
        return $this->hasMany(Order::class, 'customer_id');
    }

    public function car()
    {
        return $this->hasOne(Car::class, 'laundry_id');
    }

    function generateToken()
    {
        $this->api_token = str_random(120);
        $this->save();
        return $this->api_token;
    }

    public function generateActivation($user)
    {
        $generated_password = random_int(11111, 99999);//use it after run SMS
        /*        sendSMS($generated_password);*/
        $verification = new Verification();
        $verification->code = $generated_password; //for testing
        $user->verification()->save($verification);
        $this->sendSMS($user, $generated_password);
    }

    public function sendSMS($user, $generated_password)
    {
        $client = new Client(); //GuzzleHttp\Client

        $username = "966560705959";
        $password = "12345678";
        $number = $user->phone;
        /*        $sender = "Mghasel";*/
        $sender = "HISMS.WS-AD";
        $message = "رمز التحقق الخاص بك هو: $generated_password";

        $headers = ['Content-type' => 'text/plain; charset=utf-8'];

        $result = $client->get("https://www.hisms.ws/api.php?send_sms&username=$username&password=$password&numbers=966$number&sender=$sender&message=$message", ['headers' => $headers]);
        return $result->getStatusCode();
    }

        public function laundryRate()
    {
        return $this->hasMany(Rating::class, 'laundry_id');
    }

    public function userRatings(){
        return $this->hasMany(Rating::class, 'user_id');
    }

    public function laundryClothesPrices()
    {
return $this->hasMany(LaundryClothPrice::class,'laundry_id');
    }

    public function laundryAdditionalPrices()
    {
        return $this->hasMany(LaundryAdditionalPrice::class,'laundry_id');
    }

    public function rate()
    {
       /* return $this->orderitems->reduce(function ($total, $orderitem) {
            return $total + $orderitem->tax;
        },0);*/
       /*
        @if($item->role == 2)
                    <td>
                        @if($controller->getRate($item->id) != null)
                            @for ($i = $controller->getRate($item->id); $i >0; $i--)
                                <span class="fa fa-star"></span>
                            @endfor
                            {{--@while($value = $controller->getRate($item->id) != 0)
                                <span class="fa fa-star"></span>
                                {{$value--}}{{--
                            @endwhile--}}
                        @else <span>لم يتم تقييمه</span> @endif
                    </td>
                @endif
        * */
        /*$ratings = Rating::where('laundry_id', '=', $this->id)->get();
        if ($ratings->count() == null) {
            return null;
        }
        $total_ratings = 0;
        foreach ($ratings as $rating) {
            $total_ratings += $rating->rate;
        }
        return $total_ratings / $ratings->count();*/
       /* public
        function getTotalPrice()
        {
            return $this->buyDetails->sum(function ($buyDetail) {
                return $buyDetail->quantity * $buyDetail->price;
            });
        }*/
/*        return $this->laundryRate()->sum(DB::raw('rate'));*/
      /* return $this->laundryRate()->sum(function ($rate) {
           return $rate->rate;
       });*/
    }
    public function has_rate(){
        return $this->hasOne(Rating::class,'laundry_id');
    }
}
