<?php

namespace App\Http\Controllers\Api;

use App\Car;
use App\Location;
use App\SubscriptionOrder;
use App\SubscriptionPlan;
use App\User;
use App\UserProfile;
use App\UserWithToken;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class ApiUserController extends Controller
{
    public function store(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'name' => 'required|string',
                'phone' => ['required', 'numeric', 'unique:users', 'regex:/^(009665|9665|\+9665|05|5)(5|0|3|6|4|9|1|8|7)([0-9]{7})$/'],
                'password' => 'required|max:250|string|min:6',
                'email' => 'required|max:250|email|string|unique:users_profiles',
                'address' => 'required|max:250|string',
                'role' => ['numeric', 'required', 'regex:/[2-3]/'],
                'lat' => 'required|numeric',
                'long' => 'required|numeric',
                'company' => 'string',
                'class_model' => 'string',
                'number' => 'numeric',
                'color' => 'string',
                'driver_name' => 'string',
                'driver_phone' => 'numeric',
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $validator->errors()));
            }
            //store new user
            $user = new User();
            $user->phone = $request->phone;
            $user->password = bcrypt($request->password);
            $user->role = $request->role;
            if ($user->save()) {
                //store location
                $location = new Location();
                $location->lat = $request->lat;
                $location->long = $request->long;
                if (!($user->location()->save($location))) {
                    $user->delete();
                    return response()->json($this->result(false, "حدث خطأ غير متوقع : أثناء إضافة  الموقع الجغرافي للمستخدم", -2));
                }
                //store user profile
                $user_profile = new UserProfile();
                $user_profile->name = $request->name;
                $user_profile->email = $request->email;
                $user_profile->address = $request->address;
                if (!($user->profile()->save($user_profile))) {
                    $user->delete();
                    return response()->json($this->result(false, "حدث خطأ غير متوقع أثناء عملية التسجيل", -2));
                }
                //store laundry subscription to subscriptions
                if ($request->role == 2) {
                    $validator = Validator::make($request->all(), [
                        'plan_id' => 'required|numeric',
                        'company' => 'required|string',
                        'class_model' => 'required|string',
                        'number' => 'required|numeric',
                        'color' => 'required|string',
                        'driver_name' => 'required|string',
                        'driver_phone' => ['required', 'numeric', 'regex:/^(009665|9665|\+9665|05|5)(5|0|3|6|4|9|1|8|7)([0-9]{7})$/'],
                    ]);
                    if ($validator->fails()) {
                        $user->delete();
                        return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $validator->errors()));
                    }
                    //check if plan_id is valid
                    $plan = SubscriptionPlan::find($request->plan_id);
                    if (!$plan) {
                        $user->delete();
                        return response()->json($this->result(false, "حدث خطأ غير متوقع :تأكد من اختبار الخطة بالشكل الصحيح", -3));
                    }
                    $subscription_order = new SubscriptionOrder();
                    $subscription_order->laundry_id = $user->id;
                    $subscription_order->subscription_plan_id = $plan->id;
                    if (!($subscription_order->save())) {
                        $user->delete();
                        return response()->json($this->result(false, "حدث خطأ غير متوقع : أثناء إضافة طلب الإشتراك", -3));
                    }
                    //store laundry car details
                    $car = new Car();
                    $car->laundry_id = $user->id;
                    $car->company = $request->company;
                    $car->class_model = $request->class_model;
                    $car->number = $request->number;
                    $car->color = $request->color;
                    $car->driver_name = $request->driver_name;
                    $car->driver_phone = $request->driver_phone;
                    if (!($car->save())) {
                        $user->delete();
                        return response()->json($this->result(false, "حدث خطأ غير متوقع : أثناء إضافة تفاصيل السيارة", -3));
                    }
                }
                return $this->registered($request, $user)
                    ?: response()->json($this->result(false, "حدث خطأ غير متوقع : أثناء عملية التسجيل", -2));

            } else {
                return response()->json($this->result(false, "حدث خطأ غير متوقع : أثناء عملية التسجيل", -2));
            }
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خطأ غير متوقع : أثناء عملية التسجيل", -2));
        }
    }
    /**
     * The user has been registered.
     *
     * @param  \Illuminate\Http\Request $request
     * @param  mixed $user
     * @return mixed
     */
    protected function registered(Request $request, $user)
    {
        $user->generateActivation($user);
        return response()->json($this->result(true, "تمت عملية تسجيلك بنجاح، تم إرسال رسالة تأكيد لرقم جوالك، أدخل الرمز لإنهاء عملية التسجيل", 1));
    }

    public function verify(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'phone' => ['required', 'regex:/^(009665|9665|\+9665|05|5)(5|0|3|6|4|9|1|8|7)([0-9]{7})$/'],
                'code' => 'required|numeric',
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $validator->errors()));
            }
            //check if user found with request phone
            $user = User::where('phone', '=', $request->phone)->first();
            if ($user == null) {
                return response()->json($this->result(false, "تعذر الحصول على المستخدم", -3));
            }
            //check if user is a customer
            if ($user->role != 3) {
                return response()->json($this->result(false, "خدمة التحقق متوفرة فقط للزبائن ", -4));
            }
            //check if user verified before
            if ($user->verification->is_verified == 1) {
                return response()->json($this->result(false, "تم التحقق من الحساب مسبقاً ", 2));
            }
            //check if code is valid
            if ($user->verification->code == $request->code) {
                $user->verification->is_verified = 1;
                $user->verification->save();
                return response()->json($this->result(true, "تم التحقق من الحساب بنجاح ", 1));
            } else {
                return response()->json($this->result(false, "الرمز المدخل غير صحيح، تأكد من الرمز مرة أخرى ", -1));
            }
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء عملية تأكيد الرمز، حاول في وقت آخر ", -2));
        }
    }

   /* protected function guard()
    {
        return Auth::guard();
    }*/

    public function logout(Request $request){
        $user = Auth::guard('api')->user();
        if ($user) {
            $user->api_token = null;
            $user->save();
        }
        return response()->json($this->result(true, "تم تسجيل الخروج بنجاح", 1));
    }
    /*protected function attemptLogin(Request $request)
    {
        return $this->guard()->attempt(
            $this->credentials($request), $request->has('remember')
        );
    }*/

}
